<?php
/*
Plugin Name: Slugger
Plugin URI:	http://blog.saustrup.net/files/wp/slugger/
Description: Fixes problem with poorly translated Scandinavian and possibly German characters in post/page names (permalinks). This version is tested with WordPress 2.5 and requires PHP5.
Author: Hans Christian Saustrup
Version: 0.6
Author URI: http://blog.saustrup.net/

	CHANGES
	-------
	2006-06-29	0.1		Created
	2006-06-30	0.3		Limited functionality to save/update
	2008-04-17	0.6		Rewrote everything

	ABOUT
	-----
	The "Sanitize Title" filter in WordPress is faulty. It is supposed to translate the
	post name to pure ASCII, but either mistranslates accented characters by simply stripping
	the accents or refusing to convert them at all.

	This plugin will correctly translate most of the Scandinavian and possibly German
	characters to their correct ASCII equivalent.
	
	Submitting an article with Danish or Swedish characters previously
	resulted in a post_name (slug) like this:

		fl%c3%b8desauce-med-r%c3%a6vehaler-%c3%a6kelt-%c3%b8v
	
	With this plugin, the slug looks like this:

		floedesauce-med-raevehaler-aekelt-oev

	NOTICE
	------
	This plugin requires PHP5 and is only tested with WordPress 2.5.

*/

class WPPlugin_Saustrup_Slugger {

	private function debug($s) {
		error_log('[SluggerPlugin] '.$s);
	}

	private function formatString($s) {
		$res = '';
		for ($i=0;$i<strlen($s);$i++) $res .= ((ord($s[$i])>127)||(ord($s[$i])<32) ? '<'.ord($s[$i]).'>' : $s[$i]);
		return $res;
	}

	public function filterSanitizeTitle($title) {
		// Not UTF8? Bail out
		if (!seems_utf8($title)) return $title;
		// $this->debug('Got "'.$this->formatString($title).'"');
		$filterArr = array(
			# Single UTF8 to multiple ASCII:
			chr(195).chr(132) => 'AE',	# Auml
			chr(195).chr(164) => 'ae',	# auml
			chr(195).chr(150) => 'OE',	# Ouml
			chr(195).chr(182) => 'oe',	# ouml
			chr(195).chr(134) => 'AE',  # Aelig	
			chr(195).chr(166) => 'ae',  # aelig
			chr(195).chr(184) => 'oe',  # oslash
			chr(195).chr(152) => 'OE',  # Oslash
			chr(195).chr(133) => 'AA',  # Aring
			chr(195).chr(165) => 'aa'   # aring
		);
		// Translate and do debugging
		$filtered = strtr($title,$filterArr);
		if ($filtered!=$title) $this->debug('Translated "'.$this->formatString($title).'" to "'.$this->formatString($filtered).'"');
		// Return translated result
		return $filtered;
	}

	private function getPosts() {
		$badpattern = '%\%%';
		$posts = $this->db->get_results(
			'SELECT id,post_name,post_title '.
			'FROM `'.$this->db->posts.'` '.
			'WHERE post_name LIKE "'.$badpattern.'"'
		);
		return $posts;
	}

	private function showPostList() {
		$posts = $this->getPosts();

		// No matches?
		if (!$posts) {
			?>
			<p> No mistranslated post/page names were found. </p>
			<?
			return;
		}

		// Iterate through result
		?>
		<p> Found these potentially mistranslated post/page names: </p>
		<div class="wrap">
		<table class="widefat">
		<tr>
			<th> ID </th>
			<th> Post Title </th>
			<th scope="col"> Current Post Name </th>
			<th scope="col"> Suggested Post Name </th>
		</tr>
		<?
		foreach ($posts as $post) {
			$filtered = $this->filterSanitizeTitle(urldecode($post->post_name));
			$link = get_permalink($post->id);
			?>
			<tr>
				<td><?= $post->id ?></td>
				<td><?= htmlspecialchars($post->post_title) ?></td>
				<td><a href="<?=$link?>"><?= htmlspecialchars($post->post_name) ?></a></td>
				<td><?= htmlspecialchars($filtered) ?></td>
			</tr>
			<?
		}
		?>
		</table>
		</div>
		<form method=post>
			<input type="submit" name="slugger_repair" value="Repair mistranslated post/page names"
				onclick="return confirm('Repairing post/page names will change your permalinks. Are you sure you want to do this?')">
		</form>
		<?
	}

	private function repairPosts() {
		$count = 0;
		$posts = $this->getPosts();
		foreach ($posts as $post) {
			$filtered = $this->filterSanitizeTitle(urldecode($post->post_name));
			if ($filtered!=$post->post_name) {
				$this->debug('Repaired post #'.$post->id.': "'.$this->formatString($post->post_name).'" to "'.$this->formatString($filtered).'"');
				$this->db->query(
					'UPDATE `'.$this->db->posts.'` '.
					'SET post_name="'.$this->db->escape($slug).'" '.
					'WHERE `id`='.$post->id
				);
				$count++;
			}
		}
		if ($count) {
			?><p> Repaired <?=$count?> posts. </p><?
		} else {
			?><p> Unable to repair any posts. </p><?
		}
	}

	public function optionsPage() {
		?>   
		<div class="wrap">
		<h2><?php _e('Slugger') ?></h2>
		<p>
			Congratulations on installing the Slugger plugin. Slugger fixes some of the known problems with
			automatic post/page title to post/page name (permalink) translation - mostly in Scandinavian languages. 
			If Slugger finds any previously mistranslated post/page names, they will be listed on this page.
		</p>
		<? if(array_key_exists('slugger_repair',$_POST)) $this->repairPosts() ?>
		<? $this->showPostList() ?>
		<p>
			If you have any feedback, feel free to <a href="mailto:hc@saustrup.net">e-mail</a> the author.
		</p>
		</div>  
		<?
	}

	public function actionAdminMenu() {
		add_options_page('Slugger','Slugger',10,'',array($this,'optionsPage'));
	}

	public function __construct( $wpdb ) {
		$this->db = $wpdb; // Accessing global variables is such a filthy habit
		add_action('admin_menu',array($this,'actionAdminMenu'));
		add_filter('sanitize_title',array($this,'filterSanitizeTitle'),0);
	}
}


new WPPlugin_Saustrup_Slugger( $wpdb );

